/*
 * Copyright (c) 1997-1999 Tycho Nightingale.  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The origin of this software must not be misrepresented; you must 
 *    not claim that you wrote the original software.  If you use this 
 *    software in a product, an acknowledgment in the product 
 *    documentation would be appreciated but is not required.
 *
 * 4. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 5. The name of the author may not be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * @(#)dcddef.h	1.8 03/25/99
 */
#ifndef DCDDEF_H
#define DCDDEF_H

/*
 * Include files
 */

/*
 * Macro definitions
 */
#define TRUE		1
#define FALSE		0

#define LOG		if (Gdebug_level) dcd_log

/*
 * Be sure and check the /usr/include/sys/cmn_err.h header file
 * when determinning the constants below, because they must
 * not conflict with CE_CONT, CE_NOTICE, CE_WARN, CE_PANIC etc
 */
#define DEBUG_ERROR		(1 << 3)
#define DEBUG_VERBOSE		(1 << 4)
#define DEBUG_ENTRY		(1 << 5)
#define DEBUG_EXIT		(1 << 6)
#define DEBUG_ATTACH		(1 << 7)
#define DEBUG_DETACH		(1 << 8)
#define DEBUG_OPEN		(1 << 9)
#define DEBUG_CLOSE		(1 << 10)
#define DEBUG_STRATEGY		(1 << 11)

#define HASH_SIZE		1023
#define RAM_SEGMENTS		2
#define DISK_SEGMENTS		512 /* 128 = ~16mb, 512 = ~64mb */

#define LINE_BLOCKS		8
#define SLOT_BLOCKS		2
#define SEGMENT_SLOTS		128 /* 128 = 128kb */
#define SLOT_HIGH_WATERMARK	96

#define LARGE_REQUEST_BLOCKS	64 /* 32 kb */

#define LOG_BLOCK_OFFSET	4

#define HEADER_BLOCKS	(btod(sizeof(dcd_segment_header_t)))

#define LINE_SIZE	(LINE_BLOCKS * DEV_BSIZE)
#define SLOT_SIZE	(SLOT_BLOCKS * DEV_BSIZE)
#define HEADER_SIZE	(DEV_BSIZE * HEADER_BLOCKS)
#define SEGMENT_SIZE	(SEGMENT_SLOTS * SLOT_SIZE)
#define DISK_SEG_BLOCKS	(HEADER_BLOCKS + (SEGMENT_SLOTS * SLOT_BLOCKS))
#define DISK_SEG_SIZE	(HEADER_SIZE + SEGMENT_SIZE)

#define LINE_MASK	(~(LINE_BLOCKS - 1))
#define SLOTS_PER_LINE	(LINE_SIZE / SLOT_SIZE)
#define FRAG_MASK	((1 << LINE_SIZE/SLOT_SIZE) - 1)

#define READ_IS_NOT_CONTIG	(1 << 0)
#define READ_IN_RAM		(1 << 1)
#define READ_IN_CACHE_DISK	(1 << 2)
#define READ_IN_DATA_DISK	(1 << 3)

#define DESTAGE_HIGH_WATERMARK	0.75
#define DESTAGE_FAST_TIMEOUT	(drv_usectohz(100)) /* .1 msec */
#define DESTAGE_NORMAL_TIMEOUT	(drv_usectohz(10000)) /* 10 msec */
#define DESTAGE_SLOW_TIMEOUT	(drv_usectohz(1000000)) /* 1000 msec */

#define INSTANCE(dev)		(getminor(dev))

#define CACHE_DISK_SLICE	0
#define DATA_DISK_SLICE		1
#define SD_INSTANCE		4
/* now get ready for the ugly ugly creation of the sd minor 
 * device numbers.  as far as I cane tell the format is:
 *  (sd instance << 3 | slice) */
#define CACHE_DISK_DEV(major)	(makedevice(major, \
	((SD_INSTANCE<<3)|CACHE_DISK_SLICE)))  
#define DATA_DISK_DEV(major)	(makedevice(major, \
	((SD_INSTANCE<<3)|DATA_DISK_SLICE)))

/*
 * Enums
 */

/*
 * Structure definitions
 */
typedef struct dcd_disk_segment {
  int id;
  int ram_id;
  daddr_t slot_lba_table[SEGMENT_SLOTS];
  int valid_count;
  struct dcd_disk_segment *prev, *next;
} dcd_disk_segment_t;

typedef struct dcd_ram_segment {
  int id;
  int disk_id;
  struct dcd_ram_segment *prev, *next;
} dcd_ram_segment_t;

typedef struct dcd_frag_entry {
  int bitmap;
  int lba_segment_list[SLOTS_PER_LINE];
  int lba_slot_list[SLOTS_PER_LINE];
} dcd_frag_entry_t;

typedef struct dcd_line_entry {
  daddr_t block;
  int segment;
  int slot;
  dcd_frag_entry_t *frag;
  int key; /* for the hash table */
  struct dcd_line_entry *hash_prev, *hash_next;
} dcd_line_entry_t;

typedef struct dcd_segment_header {
  int id;
  hrtime_t timestamp;
  daddr_t slot_lba_table[SEGMENT_SLOTS];
} dcd_segment_header_t;

/*
 * Global variables
 */

/*
 * External function declarations
 */

#endif /* DCDDEF_H */
